; SPRITES.ASM
; Routines for handling sprites

; draw_sprite
;	Sends a sprite to the sprite table.
;	In:	(b,c) = (x,y) coordinates of the sprite.
;		d = index of sprite.

; draw_large_sprite
;	Sends a sprite to the sprite table. Doubles it, so draws the NEXT sprite adjacent to it.
;	In:	(b,c) = (x,y) coordinates of the sprite.
;		d = index of sprite.


;==============================================================
; ADD SPRITE TO SPRITE TABLE
;==============================================================

draw_sprite:

	ld a,(sprite_limit)
	dec a
	ret z
	ld (sprite_limit),a

	ld hl,(sprite_write_a)

.ifdef LIVE_UPDATE_SPRITES
	push hl
	call vid_goto_address
	pop hl
.endif
	ld a,c

.ifdef SUPER_SIZE_ME
	sub 16
.endif


	cp $D0
	jr nz,+
	inc a
+:

.ifdef LIVE_UPDATE_SPRITES
	out (vdp_data),a
.else
	ld (hl),a
.endif

	inc hl
	ld (sprite_write_a),hl


	ld hl,(sprite_write_b)
.ifdef LIVE_UPDATE_SPRITES
	push hl
	call vid_goto_address
	pop hl
.endif


	ld a,b
.ifdef SUPER_SIZE_ME
	sub 2
.endif

.ifdef LIVE_UPDATE_SPRITES
	out (vdp_data),a
	ld a,d
	out (vdp_data),a
	inc hl
.else
	ld (hl),a
	inc hl
	ld a,d
	ld (hl),a
.endif

	inc hl
	ld (sprite_write_b),hl

	ret

;==============================================================
; ADD TWO SPRITES (SO 16x16) TO SPRITE TABLE
;==============================================================

draw_large_sprite:

	ld a,(sprite_limit)
	dec a
	ret z
	dec a
	ret z
	ld (sprite_limit),a


.ifdef LIVE_UPDATE_SPRITES
	ld hl,(sprite_write_a)
	push hl
	call vid_goto_address
	pop hl
	inc hl
	inc hl
	ld (sprite_write_a),hl
.endif	
	ld a,c

.ifdef SUPER_SIZE_ME
	sub 16
.endif

	cp $D0
	jr nz,+
	inc a
+:

.ifdef LIVE_UPDATE_SPRITES
	out (vdp_data),a
	out (vdp_data),a
.else



	ld hl,(sprite_write_a)
	ld (hl),a
	inc hl
	ld (hl),a
	inc hl
	ld (sprite_write_a),hl
.endif


.ifdef LIVE_UPDATE_SPRITES
	ld hl,(sprite_write_b)
	push hl
	call vid_goto_address
	pop hl
	inc hl
	inc hl
	inc hl
	inc hl
	ld (sprite_write_b),hl
.endif
	

	ld a,b
.ifdef SUPER_SIZE_ME
	sub 8
.endif

.ifdef LIVE_UPDATE_SPRITES
	out (vdp_data),a
	ld a,d
	out (vdp_data),a
	ld a,b
	add a,8
	out (vdp_data),a
	ld a,d
	add a,2
	out (vdp_data),a
.else
	ld hl,(sprite_write_b)
	ld (hl),a
	inc hl
	ld a,d
	ld (hl),a
	inc hl
	ld a,b
	add a,8
	ld (hl),a
	inc hl
	ld a,d
	add a,2
	ld (hl),a
	inc hl
	ld (sprite_write_b),hl
.endif
	ret

;==============================================================
; JUMP TO START OF SPRITE TABLE
;==============================================================

reset_sprites:
	ld a,63
	ld (sprite_limit),a
	
.ifdef LIVE_UPDATE_SPRITES
	ld hl,vram_sprites
.else
	ld hl,sprite_shad_a
.endif
	ld (sprite_write_a),hl

.ifdef LIVE_UPDATE_SPRITES
	ld hl,vram_sprites+128
.else
	ld hl,sprite_shad_b
.endif

	ld (sprite_write_b),hl
	ret

;==============================================================
; WRITE TERMINATING DATA TO SPRITE TABLE
;==============================================================

finish_sprites:

	ld hl,(sprite_write_a)
.ifdef LIVE_UPDATE_SPRITES
	call vid_goto_address
	ld a,$D0
	out (vdp_data),a
.else
	ld a,$D0
	ld (hl),a
.endif
	ret

;==============================================================
; COPY SPRITE SHADOW TABLE TO REAL VRAM TABLE
;==============================================================

display_sprites:

	ld hl,vram_sprites
	call vid_goto_address
	
	call write_sprites_a

	push bc
	ld hl,vram_sprites+128
	call vid_goto_address
	pop bc

write_sprites_b:
	ld hl,sprite_shad_b

-:	ld a,(hl)
	out (vdp_data),a
	inc hl
	ld a,(hl)
	out (vdp_data),a
	inc hl
	djnz -
	ret

write_sprites_a:
	ld hl,sprite_shad_a
	ld b,0

-:	ld a,(hl)
	inc hl
	out (vdp_data),a
	inc b
	cp $D0
	jr nz,-
	ret